/*
 * Example for ccRF 3 click
 *
 * Date         Mar 2017
 * Author       Stefan Popovic
 * Copyright    GNU Public License v2
 *
 * Test configuration PIC32 :
 *  MCU           :        P32MX795F512L
 *  Dev. Board    :        EasyPIC Fusion v7
 *  SW            :        mikroC PRO for PIC32 v3.6.0
 *
 *  NOTE          : - Place ccRF 3 click board on mikroBUS slot 1
 *                  - Connect easyTFT and turn backlight ON ( turn SW11.1 )
 *                  - Pull-down PORTB pin RB0, RB1 and RB4 and turn on Button 
 *                    Press Level switch SW10 for PORTB/L (board specific)
 *                  - This example demonstrates sending and receiving packets,
 *                    with the frequency of 433 MHz of fixed size and shows
 *                    their number inside the buffer
 *                  - In order for packet to be successfuly transmitted and
 *                    received, one should be in Rx mode, while other in Tx
 *                  - Use button RB0 for initialization, RB1 for sending
 *                    and RB4 for receiving packets
 *
/*******************************************************************************
* Includes
*******************************************************************************/
#include "tft_resources.h"
#include "ccRF_3_resources.h"
#include "easyLink.h"
#include <stdbool.h>

/*******************************************************************************
* Module Preprocessor Constants
*******************************************************************************/

/*******************************************************************************
* Module Variable Definitions
*******************************************************************************/
sbit CCRF_3_RST             at LATC1_bit;
sbit CCRF_3_RST_DIR         at TRISC1_bit;
sbit CCRF_3_CS              at LATC2_bit;
sbit CCRF_3_CS_DIR          at TRISC2_bit;
sbit CCRF_3_GPO             at RB8_bit;
sbit CCRF_3_GPO_DIR         at TRISB8_bit;
sbit CCRF_3_GP2             at RD0_bit;
sbit CCRF_3_GP2_DIR         at TRISD0_bit;
sbit CCRF_3_GP3             at RE8_bit;
sbit CCRF_3_GP3_DIR         at TRISE8_bit;

bool init_done_f        = false;
bool init_f             = false;
bool send_f             = false;
bool receive_f          = false;
bool reading_f          = false;
/*******************************************************************************
* Function Declarations
*******************************************************************************/
static void check_buttons( void );
static void is_initialized( void );
static void is_sending_mode( void );
static void is_receiving_mode( void );
static void system_init( void );
/*******************************************************************************
* Function Definitions
*******************************************************************************/
 /*
 * Checking Button Pressed Events
 *
 * This functions checks if one of the appointed buttons have been pressed
 */
static void check_buttons( void )
{
 if (Button(&PORTB, 0, 100, 1))
 {
   if (!init_done_f)
  {
    init_f = true;
  }
 }
 else if (Button(&PORTB, 1, 100, 1))
 {
  if (init_done_f)
  {
   send_f = true;
  }
 }
 else if (Button(&PORTB, 4, 100, 1))
 {
  if (init_done_f)
  {
   receive_f = true;
  }
 }
}
 /*
 * Checking whether necessary initialization has been performed
 */
static void is_initialized( void )
{
 if ( init_f )
    {
     init_f = false;
     ccRF_Configure();
     TFT_Rectangle(15,75,300,120);
     TFT_Write_Text("Initialized.", 15, 80);
     init_done_f = 1;
    }
}
/*
 * Checking if sending mode has been requested
 */
static void is_sending_mode( void )
{
 if ( send_f )
    {
     send_f = false;
     reading_f = false;
     Delay_ms(200);
     ccRF_RunTX();
     TFT_Write_Text("Sending...", 15, 130);
    }
}
/*
 * Checking if receiving mode has been requested
 */
static void is_receiving_mode( void )
{
 if ( receive_f )
    {
     receive_f = false;
     ccRF_SetRXmode();
     TFT_Write_Text("Receiving...", 15, 180);
     reading_f = true;
    }
}
/*
 * System Initialization
 *
 * Configuring MCU ports, initializing SPI module and performing reset
 */
static void system_init( void )
{
    // Configuring GPIO pins
    AD1PCFG = 0xFFFF;            // Configure AN pins as digital I/O
    JTAGEN_bit = 0;              // Disable JTAG
    CCRF_3_RST_DIR = 0;
    CCRF_3_CS_DIR  = 0;
    CCRF_3_GP3_DIR = 1;
    CCRF_3_GPO_DIR = 1;
    CCRF_3_GP2_DIR = 1;
    
    TRISB0_bit = 1;
    TRISB1_bit = 1;
    TRISB4_bit = 1;
    TRISB8_bit = 1;
    
    // SPI Initialization
    SPI3_Init_Advanced( _SPI_MASTER, _SPI_8_BIT, 16, _SPI_SS_DISABLE,
                    _SPI_DATA_SAMPLE_MIDDLE, _SPI_CLK_IDLE_LOW,
                    _SPI_IDLE_2_ACTIVE );
    Delay_ms( 100 );

    // Perform ccRF 3 board reset
    CCRF_3_RST = 1;
    Delay_ms( 100 );
    CCRF_3_RST = 0;
    Delay_ms( 100 );
    CCRF_3_RST = 1;
    CCRF_3_CS = 1;
}

/*
 * Main function
 */
void main( void )
{
  system_init();
  display_init();
  TFT_Set_Font( &HandelGothic_BT21x22_Regular, CL_PURPLE, FO_HORIZONTAL );
  TFT_Write_Text("Please  initialize  the  module.", 15, 80);
  
  while (1)
  {
    check_buttons();
    is_initialized();
    is_sending_mode();
    is_receiving_mode();
    if ( reading_f )
    {
     ccRF_RunRX();
    }
  }
}
/************************************************************ END OF FILE *****/